using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using UnityEngine.AI;

public class EnemyController : MonoBehaviour
{

    Animator animator;

    int standUpKindHash;
    int walkKindHash;
    int isCloseEnoughHash;
    int enemyDeadHash;
    int playerDeadHash;

    int randomStandUp;
    int randomWalk;

    private NavMeshAgent navMeshAgent;
    GameObject movePositionTransform;
    public int maxHealth = 100;
    int currentHealth;

    public Transform attackPoint;
    public float attackDistance = 0.5f;
    public LayerMask playerLayers;
    public float attackInterval = 1.0f;
    public int attackDamage = 30;
    float nextAttack;

    private bool isPlayerDead = false;

    void Awake(){
        randomStandUp = new System.Random().Next(0, 2);
        //randomWalk = new System.Random().Next(0, 2);
        randomWalk = 1;
        navMeshAgent = GetComponent<NavMeshAgent>();
    }

    // Start is called before the first frame update
    void Start()
    {
        animator = GetComponent<Animator>();
        standUpKindHash = Animator.StringToHash("StandUpKind");
        //bool standUpAux = Convert.ToBoolean(randomStandUp);
        animator.SetBool(standUpKindHash, Convert.ToBoolean(randomStandUp));
        isCloseEnoughHash = Animator.StringToHash("isCloseEnough");
        enemyDeadHash = Animator.StringToHash("enemyDead");
        walkKindHash = Animator.StringToHash("walkKind");
        playerDeadHash = Animator.StringToHash("playerDead");
        animator.SetBool(walkKindHash, Convert.ToBoolean(randomWalk));
        navMeshAgent.isStopped = true;
        currentHealth = maxHealth;

        movePositionTransform = GameObject.FindWithTag("Player");

    }

    /*void OnColliderStay(Collider other){
        if(other.CompareTag("ExtrudedStructure") || other.CompareTag("ModeledStructure") || other.CompareTag("AreaWater")){
            Destroy(this);
            Debug.Log("enemy destroyed");
        }
    }*/

    void OnTriggerStay(Collider other){
        if(other.CompareTag("ExtrudedStructure") || other.CompareTag("ModeledStructure") || other.CompareTag("AreaWater")){
            animator.enabled = false;
            navMeshAgent.enabled = false;
            Destroy(gameObject);
            Debug.Log("enemy destroyed");
        }
    }

    public void enableFollowing(){
        if(!isPlayerDead) navMeshAgent.isStopped = false;
    }

    public void TakeDamage(int damage){
        currentHealth -= damage;

        if(currentHealth <= 0){
            Die();
        }
    }

    void Die(){
        navMeshAgent.isStopped = true;
        animator.SetTrigger(enemyDeadHash);
        GetComponent<Collider>().enabled = false;
        GetComponent<Rigidbody>().detectCollisions = false;
        Destroy(gameObject, 5.0f);
        //this.enabled = false;
    }

    public void playerDead(){
        animator.SetTrigger(playerDeadHash);
        //NavMeshAgent navMeshAgent = GetComponent<NavMeshAgent>();
        //GetComponent<Rigidbody>().detectCollisions = false;
        GetComponent<Collider>().enabled = false;
        GetComponent<Rigidbody>().detectCollisions = false;
        isPlayerDead = true;
        navMeshAgent.isStopped = true;
        navMeshAgent.enabled = false;
        //this.enabled = false;
    }

    void Attack(){
        bool isCloseEnough = animator.GetBool(isCloseEnoughHash);
        float distanceToPlayer = Vector3.Distance(this.transform.position, movePositionTransform.transform.position);
        //Debug.Log(distanceToPlayer);
        if(Time.deltaTime > nextAttack && !isCloseEnough &&  distanceToPlayer < attackDistance){
            nextAttack = Time.deltaTime + attackInterval;
            Collider[] hitPlayer = Physics.OverlapSphere(attackPoint.position, attackDistance, playerLayers);
            foreach(Collider player in hitPlayer){
                player.GetComponent<PlayerAttack>().TakeDamage(attackDamage);
            }
            animator.SetBool(isCloseEnoughHash, true);
            if(!isPlayerDead) navMeshAgent.isStopped = true;
        }

        if(isCloseEnough && distanceToPlayer > attackDistance){
            animator.SetBool(isCloseEnoughHash, false);
            navMeshAgent.isStopped = false;
        }
    }

    // Update is called once per frame
    void Update()
    {
        if(!isPlayerDead){
            navMeshAgent.destination = movePositionTransform.transform.position;
            Attack();
        }
        
        //Debug.Log(movePositionTransform.transform.position);
    }
}
