using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;


public class MovementAndAnimationPlayerController : MonoBehaviour
{
    //variables globals
    PlayerInput playerInput;
    CharacterController characterController;
    bool isGrounded;
    float gravityValue = -9.81f;
    //public float jumpHeight = 0.5f;
    public float jumpSpeed = 8f;

    public GameObject freeCamera;

    Vector2 currentMovementInput;
    //Vector2 currentMouseInput;
    Vector3 currentMovement;
    //Vector3 currentRotation;
    Vector3 currentRunMovement;
    bool isMovementPressed;

    Animator animator;
    public float acceleration = 2.0f;
    public float deceleration = 2.0f;
    public float maximumWalkVelocity = 0.5f;
    public float maximumRunVelocity = 2.0f;
    //public float aimAcceleration = 0.05f;
    public float walkSpeedMultiplier = 1.5f;
    public float runMultiplier = 3.0f;
    public float runAnimationMultiplier = 5.0f;

    public float turnSmoothTime = 1f;
    public GameObject staticCamera;
    //public float maxRotationSpeed = 10f;
    //public float mouseSensitivity = 0.1f;
    //float KeyRotationFactorPerFrame = 1.0f;
    //float MouseRotationFactorPerFrame = 5.0f;
    float velocityZ = 0.0f;
    float velocityX = 0.0f;
    //float aiming = 0.0f;
    bool jumpPressed;
    //bool aimPressed;
    //bool attackPressed;
    bool runPressed;
    bool activeCamera; //true = static, false = free

    float turnSmoothVelocity;

    int VelocityZHash;
    int VelocityXHash;
    int jumpHash;
    //int aimHash;
    //nt attackHash;

    // Start is called before the first frame update

    void Awake(){
        Cursor.lockState = CursorLockMode.Locked;

        playerInput = new PlayerInput();
        activeCamera = true;
        characterController = GetComponent<CharacterController>();

        playerInput.CharacterControls.Move.started += onMovementInput;
        playerInput.CharacterControls.Move.performed += onMovementInput;
        playerInput.CharacterControls.Move.canceled += onMovementInput;

        /*playerInput.CharacterControls.Rotate.started += onMouseInput;
        playerInput.CharacterControls.Rotate.performed += onMouseInput;
        playerInput.CharacterControls.Rotate.canceled += onMouseInput;*/

        playerInput.CharacterControls.Jump.started += onJumpInput;
        playerInput.CharacterControls.Jump.canceled += onJumpInput;

        playerInput.CharacterControls.Run.started += onRunInput;
        playerInput.CharacterControls.Run.canceled += onRunInput;

        /*playerInput.CharacterControls.Aim.started += onAimInput;
        playerInput.CharacterControls.Aim.canceled += onAimInput;

        playerInput.CharacterControls.Attack.started += onAttackInput;
        playerInput.CharacterControls.Attack.canceled += onAttackInput;*/

        playerInput.CharacterControls.FreeCamera.started += onSetActiveStart;
        //playerInput.CharacterControls.FreeCamera.canceled += onSetActiveCancel;
    }

    void onSetActiveCancel(InputAction.CallbackContext context){

        staticCamera.SetActive(true);
        freeCamera.transform.position = staticCamera.transform.position;
        freeCamera.transform.rotation = staticCamera.transform.rotation;
        freeCamera.SetActive(false);
    }

    void onSetActiveStart(InputAction.CallbackContext context){
        if(activeCamera){
            freeCamera.SetActive(true);
            freeCamera.transform.position = staticCamera.transform.position;
            freeCamera.transform.rotation = staticCamera.transform.rotation;
            staticCamera.SetActive(false);
        }
        else{
            staticCamera.SetActive(true);
            freeCamera.transform.position = staticCamera.transform.position;
            freeCamera.transform.rotation = staticCamera.transform.rotation;
            freeCamera.SetActive(false);
        }
        activeCamera = !activeCamera;
    }

    /*void onMouseInput(InputAction.CallbackContext context){
        currentMouseInput = context.ReadValue<Vector2>();
        Debug.Log(currentMouseInput);
    }*/

    /*void onAttackInput(InputAction.CallbackContext context){
        attackPressed = context.ReadValueAsButton();
    }

    void onAimInput(InputAction.CallbackContext context){
        aimPressed = context.ReadValueAsButton();
    }*/

    void onRunInput(InputAction.CallbackContext context){
        runPressed = context.ReadValueAsButton();
    }
    
    void onJumpInput(InputAction.CallbackContext context){
        jumpPressed = context.ReadValueAsButton();
    }

    void onMovementInput(InputAction.CallbackContext context){
            currentMovementInput = context.ReadValue<Vector2>();
            //Debug.Log(currentMovementInput);
            currentMovement.x = currentMovementInput.x * walkSpeedMultiplier;
            currentMovement.z = currentMovementInput.y * walkSpeedMultiplier;
            currentRunMovement.x = currentMovementInput.x * runMultiplier;
            currentRunMovement.z = currentMovementInput.y * runMultiplier;
            isMovementPressed = currentMovementInput.x != 0 || currentMovementInput.y != 0;
    }

    void OnEnable(){
        playerInput.CharacterControls.Enable();
    }

    void OnDisable(){
        playerInput.CharacterControls.Disable();
    }

    void Start()
    {
        animator = GetComponent<Animator>();

        VelocityZHash = Animator.StringToHash("Velocity Z");
        VelocityXHash = Animator.StringToHash("Velocity X");
        jumpHash = Animator.StringToHash("wannaJump");
        /*aimHash = Animator.StringToHash("aiming");
        attackHash = Animator.StringToHash("isAttacking");*/
    }

    void changeVelocity(bool forwardPressed, bool backPressed, bool leftPressed, bool rightPressed, bool runPressed, float currentMaxVelocity){
        if(forwardPressed && velocityZ < currentMaxVelocity){
            if(runPressed){
                velocityZ += Time.deltaTime * acceleration * runAnimationMultiplier;
            }
            else{
                velocityZ += Time.deltaTime * acceleration;
            }
            
        }
        if(leftPressed && velocityX > -currentMaxVelocity){
            if(runPressed){
                velocityX -= Time.deltaTime * acceleration * runAnimationMultiplier;
            }
            else{
                velocityX -= Time.deltaTime * acceleration;
            }
        }
        if(rightPressed && velocityX < currentMaxVelocity){
            if(runPressed){
                velocityX += Time.deltaTime * acceleration * runAnimationMultiplier;
            }
            else{
                velocityX += Time.deltaTime * acceleration;
            }
        }
        if(backPressed && velocityZ > -(currentMaxVelocity/2)){
            if(runPressed){
                velocityZ -= Time.deltaTime * acceleration * runAnimationMultiplier;
            }
            else{
                velocityZ -= Time.deltaTime * acceleration;
            }
        }
    }

    void lockOrResetVelocity(bool forwardPressed, bool backPressed, bool leftPressed, bool rightPressed, bool runPressed, float currentMaxVelocity){
        if(!forwardPressed && velocityZ > 0.0f){
            if(runPressed){
                velocityZ -= Time.deltaTime * deceleration * runAnimationMultiplier;
            }
            else{
                velocityZ -= Time.deltaTime * deceleration;
            }
        }

        if(!backPressed && velocityZ < 0.0f){
            velocityZ += Time.deltaTime * deceleration;
        }

        if(!forwardPressed && !backPressed && velocityZ != 0.0f && (velocityZ > -0.05f && velocityZ < 0.05f)){
            velocityZ = 0.0f;
        }

        if(!leftPressed && velocityX < 0.0f){
            if(runPressed){
                velocityX += Time.deltaTime * deceleration * runAnimationMultiplier;
            }
            else{
                velocityX += Time.deltaTime * deceleration;
            }
            
        }
        if(!rightPressed && velocityX > 0.0f){
            if(runPressed){
                velocityX -= Time.deltaTime * deceleration * runAnimationMultiplier;
            }
            else{
                velocityX -= Time.deltaTime * deceleration;
            }
        }

        if(!leftPressed && !rightPressed && velocityX != 0.0f && (velocityX > -0.05f && velocityX < 0.05f)){
            velocityX = 0.0f;
        }

        if(forwardPressed && runPressed && velocityZ > currentMaxVelocity){
            velocityZ = currentMaxVelocity;
        }
        else if(forwardPressed && velocityZ > currentMaxVelocity){
            velocityZ -= Time.deltaTime * deceleration;
            if(velocityZ > currentMaxVelocity && velocityZ < (currentMaxVelocity + 0.05f)){
                velocityZ = currentMaxVelocity;
            }
        }
        else if(forwardPressed && velocityZ < currentMaxVelocity && velocityZ >(currentMaxVelocity - 0.05f)){
            velocityZ = currentMaxVelocity;
        }

        if(backPressed && runPressed && velocityZ < -(currentMaxVelocity/2)){
            velocityZ = -(currentMaxVelocity/2);
        }
        else if(backPressed && velocityZ < -(currentMaxVelocity/2)){
            velocityZ += Time.deltaTime * deceleration;
            if(velocityZ < currentMaxVelocity && velocityZ > (-(currentMaxVelocity/2) - 0.05f)){
                velocityZ = -(currentMaxVelocity/2);
            }
        }
        else if(backPressed && velocityZ > -(currentMaxVelocity/2) && velocityZ < (-(currentMaxVelocity/2) + 0.05f)){
            velocityZ = -(currentMaxVelocity/2);
        }

        if(leftPressed && runPressed && velocityX < -currentMaxVelocity){
            velocityX = -currentMaxVelocity;
        }
        else if(leftPressed && velocityX < -currentMaxVelocity){
            velocityX += Time.deltaTime * deceleration;
            if(velocityX < -currentMaxVelocity && velocityX > (-currentMaxVelocity - 0.05f)){
                velocityX = -currentMaxVelocity;
            }
        }
        else if(leftPressed && velocityX > -currentMaxVelocity && velocityX < (-currentMaxVelocity + 0.05f)){
            velocityX = -currentMaxVelocity;
        }

        if(rightPressed && runPressed && velocityX > currentMaxVelocity){
            velocityX = currentMaxVelocity;
        }
        else if(rightPressed && velocityX > currentMaxVelocity){
            velocityX -= Time.deltaTime * deceleration;
            if(velocityX > currentMaxVelocity && velocityX < (currentMaxVelocity + 0.05f)){
                velocityX = currentMaxVelocity;
            }
        }
        else if(rightPressed && velocityX < currentMaxVelocity && velocityX >(currentMaxVelocity - 0.05f)){
            velocityX = currentMaxVelocity;
        }
    }

    void checkJumping(){
        bool isJumping = animator.GetBool(jumpHash);
        if(jumpPressed && !isJumping){
            animator.SetBool(jumpHash, true);
        }
        if(!jumpPressed && isJumping){
            animator.SetBool(jumpHash, false);
        }
    }

    /*void changeAiming(float attackLayerWeight){
        if(aimPressed && aiming < 1.0f){
            if(attackLayerWeight < 1.0f){animator.SetLayerWeight(animator.GetLayerIndex("meleeAttack"), 1.0f);}
            aiming += aimAcceleration * Time.deltaTime;
            //Debug.Log(aimPressed);
        }
    }

    void lockOrResetAiming(float attackLayerWeight){
        if(!aimPressed && aiming > 0.0f){
            aiming -= aimAcceleration * Time.deltaTime;
            if(aiming<0.0f){aiming = 0.0f;}
        }
        if(aiming == 0.0f) {animator.SetLayerWeight(animator.GetLayerIndex("meleeAttack"), 0.0f);}
    }

    void checkAttack(bool attacking){
        if(aimPressed && attackPressed && !attacking){
            animator.SetBool(attackHash, true);
        }
    }

    void lockOrResetAttacking(bool attacking){
        if(!attackPressed && attacking){
            animator.SetBool(attackHash, false);
        }
    }*/

    void handleRotation(){
        if(currentMovementInput.magnitude >= 0.1f){
            float targetAngle = Mathf.Atan2(currentMovementInput.x, currentMovementInput.y) * Mathf.Rad2Deg + staticCamera.transform.eulerAngles.y;
            float angle = Mathf.SmoothDampAngle(transform.eulerAngles.y, targetAngle, ref turnSmoothVelocity, turnSmoothTime);
            transform.rotation = Quaternion.Euler(0f, angle, 0f);
            //float keyInputAngle = Mathf.Atan2(currentMovementInput.x, currentMovementInput.y) * Mathf.Rad2Deg + cam.transform.eulerAngles.y;
            currentMovement = Quaternion.Euler(0f, targetAngle, 0f) * Vector3.forward * walkSpeedMultiplier;
            currentRunMovement = currentMovement * runMultiplier;
            
        }

        /*if(currentMouseInput.magnitude >= 0.1f){
            currentRotation.y += currentMouseInput.x * Time.deltaTime * mouseSensitivity;
            transform.rotation = Quaternion.Euler(0f, currentRotation.y, 0f);
        }*/

        /*if(rightPressed && maxRotationSpeed > currentRotation.y){
            currentRotation.y += KeyRotationFactorPerFrame * Time.deltaTime;
        }
        else if(leftPressed && -maxRotationSpeed < currentRotation.y){
            currentRotation.y -= KeyRotationFactorPerFrame * Time.deltaTime;
        }
        else{
            currentRotation.y = 0.0f;
        }
        transform.Rotate(currentRotation, Space.Self);*/
    }

    void handleJumpsAndGravity(){
        isGrounded = characterController.transform.position.y < 0.30f;
        //Debug.Log(isGrounded);
        float ySpeed = currentMovement.y;
        ySpeed += gravityValue * Time.deltaTime;
        if(isGrounded){
            ySpeed = -0.5f;
            if(jumpPressed){
                ySpeed = jumpSpeed;
            }
        }
        //Debug.Log(ySpeed);
        currentMovement.y = ySpeed;
        currentRunMovement.y = ySpeed;
    }

    public void EnableMovement(){
        characterController.enabled = true;
    }

    public void DisableMovement(){
        characterController.enabled = false;
    }

    public void UnlockCursor(){
        Cursor.lockState = CursorLockMode.None;
    }

    // Update is called once per frame
    void Update()
    {
        bool forwardPressed = currentMovementInput.y > 0.0f;
        bool backPressed = currentMovementInput.y < 0.0f;
        bool leftPressed = currentMovementInput.x < 0.0f;
        bool rightPressed = currentMovementInput.x > 0.0f;

        float currentMaxVelocity = runPressed ? maximumRunVelocity : maximumWalkVelocity;
        //float attackLayerWeight = animator.GetLayerWeight(animator.GetLayerIndex("meleeAttack"));

        //bool attacking = animator.GetBool(attackHash);
        
        changeVelocity(forwardPressed, backPressed, leftPressed, rightPressed, runPressed, currentMaxVelocity);
        lockOrResetVelocity(forwardPressed, backPressed, leftPressed, rightPressed, runPressed, currentMaxVelocity);
        checkJumping();
        /*changeAiming(attackLayerWeight);
        lockOrResetAiming(attackLayerWeight);
        checkAttack(attacking);
        lockOrResetAttacking(attacking);*/
        
        animator.SetFloat(VelocityZHash, velocityZ);
        animator.SetFloat(VelocityXHash, velocityX);
        //animator.SetFloat(aimHash, aiming);

        handleRotation();
        handleJumpsAndGravity();

        //Debug.Log(currentRunMovement.y);
        if(runPressed){
            characterController.Move(currentRunMovement * Time.deltaTime);
        }
        else{
            characterController.Move(currentMovement * Time.deltaTime);
        }
        

    }
}
