using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.InputSystem;


public class PlayerAttack : MonoBehaviour
{
    Animator animator;
    float aiming = 0.0f;
    bool aimPressed;
    public float aimAcceleration = 0.05f;
    bool attackPressed;
    int aimHash;
    int attackHash;
    int isDeadHash;
    public Transform attackPoint;
    public float attackRange = 0.5f;
    public LayerMask enemyLayers;
    public float attackInterval = 1.0f;
    public int meleeDamage = 30;
    float nextAttack = 0f;
    public int maxHealth = 100;
    int currentHealth;

    PlayerInput playerInput;

    private bool isDead = false;


    void Awake(){
        playerInput = new PlayerInput();
        playerInput.CharacterControls.Aim.started += onAimInput;
        playerInput.CharacterControls.Aim.canceled += onAimInput;

        playerInput.CharacterControls.Attack.started += onAttackInput;
        playerInput.CharacterControls.Attack.canceled += onAttackInput;

        playerInput.CharacterControls.PauseGame.started += onPauseInput;
        //playerInput.CharacterControls.PauseGame.canceled += onPauseInput;
    }

    void onPauseInput(InputAction.CallbackContext context){
        if(Time.timeScale == 0){
            Time.timeScale = 1;
            Debug.Log("Game unpaused");
        }
        else{
            Time.timeScale = 0;
            Debug.Log("Game paused"); 
        } 
        
    }

    void onAttackInput(InputAction.CallbackContext context){
        attackPressed = context.ReadValueAsButton();
    }

    void onAimInput(InputAction.CallbackContext context){
        aimPressed = context.ReadValueAsButton();
    }

    void changeAiming(float attackLayerWeight){
        if(aimPressed && aiming < 1.0f){
            if(attackLayerWeight < 1.0f){animator.SetLayerWeight(animator.GetLayerIndex("meleeAttack"), 1.0f);}
            aiming += aimAcceleration * Time.deltaTime;
            //Debug.Log(aimPressed);
        }
    }

    void lockOrResetAiming(float attackLayerWeight){
        if(!aimPressed && aiming > 0.0f){
            aiming -= aimAcceleration * Time.deltaTime;
            if(aiming<0.0f){aiming = 0.0f;}
        }
        if(aiming == 0.0f) {animator.SetLayerWeight(animator.GetLayerIndex("meleeAttack"), 0.0f);}
    }

    void checkAttack(bool attacking){
        if(aimPressed && attackPressed && !attacking){
            animator.SetBool(attackHash, true);
        }
    }

    void lockOrResetAttacking(bool attacking){
        if(!attackPressed && attacking){
            animator.SetBool(attackHash, false);
        }
    }



    // Start is called before the first frame update
    void Start()
    {
        animator = GetComponent<Animator>();
        aimHash = Animator.StringToHash("aiming");
        attackHash = Animator.StringToHash("isAttacking");
        isDeadHash = Animator.StringToHash("isDead");
        currentHealth = maxHealth;
    }

    // Update is called once per frame
    void Update()
    {
        if(!isDead){
            bool attacking = animator.GetBool(attackHash);
            float attackLayerWeight = animator.GetLayerWeight(animator.GetLayerIndex("meleeAttack"));

            changeAiming(attackLayerWeight);
            lockOrResetAiming(attackLayerWeight);
            checkAttack(attacking);
            lockOrResetAttacking(attacking);

            animator.SetFloat(aimHash, aiming);
            if(aimPressed && attackPressed){
                Attack();
            }
        }
    }

    void Attack(){
        if(Time.time > nextAttack){
            nextAttack = Time.time + attackInterval;
            Collider[] hitEnemies = Physics.OverlapSphere(attackPoint.position, attackRange, enemyLayers);
            foreach(Collider enemic in hitEnemies){
                enemic.GetComponent<EnemyController>().TakeDamage(meleeDamage);
                //Debug.Log(enemic.name);
            }
        }
            
    }

    void OnDrawGizmoSelected(){
        if(attackPoint == null) return;
        Gizmos.DrawSphere(attackPoint.position, attackRange);
    }

    void OnEnable(){
        playerInput.CharacterControls.Enable();
    }

    void OnDisable(){
        playerInput.CharacterControls.Disable();
    }

    public void TakeDamage(int damage){
        if(!isDead){
            Debug.Log("Player takes dmg");
            currentHealth -= damage;
            if(currentHealth <= 0){
                Die();
            }
        }
        
    }

    void Die(){
        Debug.Log("Player died");
        isDead = true;
        animator.SetTrigger(isDeadHash);
        Vector3 actualPosition = transform.position;
        //actualPosition.y = -0.75f;
        transform.position = actualPosition;
        GetComponent<MovementAndAnimationPlayerController>().enabled = false;
        GetComponent<CharacterController>().enabled = false;
        GameObject gameController = GameObject.FindWithTag("GameController");
        gameController.GetComponent<GameControllerScript>().playerDead();
        //this.enabled = false;
    }

    public void CallFadeOut(){
        GameObject gameController = GameObject.FindWithTag("GameController");
        gameController.GetComponent<GameControllerScript>().CallFadeOut();
    }

}
